SystemOrganization addCategory: #'Irka-Core'!
SystemOrganization addCategory: #'Irka-GUI'!
SystemOrganization addCategory: #'Irka-Test'!

StandardWindow subclass: #IrkaBrowser
	instanceVariableNames: 'servers chans users messagePanels messageContainer'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Irka-GUI'!

!IrkaBrowser methodsFor: 'private' stamp: 'AttilaMagyar 9/23/2012 20:03'!
createLeftRightPane
	^ Morph new
		layoutPolicy: RowLayout new;
		listDirection: #leftToRight;
		hResizing: #spaceFill;
		vResizing: #spaceFill;			
		cellInset: 1;
		layoutInset: 1;
		color: Color transparent;
		yourself! !

!IrkaBrowser methodsFor: 'private' stamp: 'AttilaMagyar 9/26/2012 20:46'!
createListBox: anIrkaListModel
	^ (PluggableListMorph 
				on: anIrkaListModel list: #list selected: #index changeSelected: #index: menu: #menu)
				hResizing: #spaceFill; 
				vResizing: #spaceFill;
				yourself! !

!IrkaBrowser methodsFor: 'private' stamp: 'AttilaMagyar 9/26/2012 20:30'!
createMessageContainer
	^ Morph new
		layoutPolicy: TableLayout new;
		color: Color transparent;
		hResizing: #spaceFill;
		vResizing: #spaceFill;
		yourself! !

!IrkaBrowser methodsFor: 'private' stamp: 'AttilaMagyar 9/26/2012 20:28'!
createTopBottomPane
	^ Morph new
		layoutPolicy: TableLayout new;
		listDirection: #topToBottom;
		hResizing: #spaceFill;
		vResizing: #spaceFill;		
		cellInset: 1;
		layoutInset: 1;		
		color: Color transparent;
		yourself.! !

!IrkaBrowser methodsFor: 'initialization' stamp: 'AttilaMagyar 9/26/2012 20:40'!
initWindow
	| contentPane browserPane controlPane serverList chanList userList |
	self title: 'Irka browser'.
	messageContainer := self createMessageContainer.
	contentPane := self createTopBottomPane.
	browserPane := self createLeftRightPane.
	controlPane := self createLeftRightPane.
	servers := IrkaListModel new.
	servers add: 'irc.freenode.net'.	"TODO"
	chans := IrkaListModel new.
	users := IrkaListModel new.
	serverList := self createListBox: servers.
	chanList := self createListBox: chans.
	userList := self createListBox: users.
	browserPane
		addMorph: userList;
		addMorph: chanList;
		addMorph: serverList.
	contentPane 
		addMorph: messageContainer;
		addMorph: browserPane.
	self addMorph: contentPane frame: (0 @ 0 corner: 1 @ 01)! !

!IrkaBrowser methodsFor: 'initialization' stamp: 'AttilaMagyar 9/24/2012 22:55'!
initialize
	super initialize.
	self initWindow.
	messagePanels := Dictionary new.! !

!IrkaBrowser methodsFor: 'events' stamp: 'AttilaMagyar 9/24/2012 23:35'!
joinedTo: anIrkaJoined
	| messagesMorph |
	"TODO only if nick = me"
	messagesMorph := IrkaMessagesMorph new.
	messagePanels at: anIrkaJoined chan put: messagesMorph.
	
	WorldState addDeferredUIMessage: [ 
		messageContainer removeAllMorphs.
		messageContainer addMorph: messagesMorph ].
	
	chans add: anIrkaJoined chan.
	anIrkaJoined chan announcer on: IrkaMessage send: #messageReceived: to: self.! !

!IrkaBrowser methodsFor: 'events' stamp: 'AttilaMagyar 9/24/2012 23:20'!
messageReceived: anIrkaMessage
	| messagesMorph |
	messagesMorph := messagePanels at: anIrkaMessage chan.
	messagesMorph addMessage: ('[', anIrkaMessage user ,']>> ', anIrkaMessage message).
	
	users clean. "TODO"
	anIrkaMessage chan users do: [:each | users add: each ].! !

BabyMockTestCase subclass: #IrkaChannelTest
	instanceVariableNames: 'chan translator'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Irka-Test'!

!IrkaChannelTest methodsFor: 'running' stamp: 'AttilaMagyar 9/23/2012 19:03'!
setUp
	translator := context mock: 'translator'.
	chan := IrkaChannel named: '#mychan' translator: translator.
	context visualize: chan.! !

!IrkaChannelTest methodsFor: 'tests' stamp: 'AttilaMagyar 9/23/2012 19:23'!
testThatItCanBeLeft
	translator should receive: #translateLeave:; with: '#mychan'.
	chan leave.! !

!IrkaChannelTest methodsFor: 'tests' stamp: 'AttilaMagyar 9/23/2012 19:03'!
testThatItCanSendMessagesToTheChan
	translator should receive: #translateMessage:to:; with: 'msg 1 2' and: '#mychan'.
	chan message: 'msg 1 2'.! !

BabyMockTestCase subclass: #IrkaSessionTest
	instanceVariableNames: 'session translator me'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Irka-Test'!

!IrkaSessionTest methodsFor: 'matchers' stamp: 'AttilaMagyar 9/16/2012 00:12'!
chan: anIrkaChannel hasUsers: anArray 
	^ anIrkaChannel users = (OrderedCollection newFrom: anArray).! !

!IrkaSessionTest methodsFor: 'private' stamp: 'AttilaMagyar 9/22/2012 23:52'!
joined: chanName
	| joinedChan |
	session announcer on: IrkaJoined do: [:act | joinedChan := act chan ].
	session translated: (IrkaTranslated joined: me to: chanName).
	^ joinedChan! !

!IrkaSessionTest methodsFor: 'running' stamp: 'AttilaMagyar 9/26/2012 22:48'!
setUp
	me := 'mynick'.
	session := IrkaSession new.
	translator := context mock: 'translator'.
	session translator: translator.
	context visualize: session.! !

!IrkaSessionTest methodsFor: 'tests' stamp: 'AttilaMagyar 9/23/2012 18:50'!
testThatItCanChangeNick
	translator should receive: #translateChangeNick:; with: 'newnick'.
	session changeNick: 'newnick'.
! !

!IrkaSessionTest methodsFor: 'tests' stamp: 'AttilaMagyar 9/22/2012 23:48'!
testThatItCanJoinToChan
	translator should receive: #translateJoin:; with: '#chan'.
	session joinChan: '#chan'.
! !

!IrkaSessionTest methodsFor: 'tests' stamp: 'AttilaMagyar 9/26/2012 19:30'!
testThatItCanQuit
	translator should receive: #translateQuit.
	session close.
! !

!IrkaSessionTest methodsFor: 'tests' stamp: 'AttilaMagyar 9/22/2012 23:45'!
testThatItForwardsIncomingMessageToChan
	| chanListener chan |		
	chanListener := context mock: 'chanListener'.	
	chanListener should receive: #msg:; with: [ :its | its message = 'msg 1 2' & its chan == chan & its user = 'nick' ].																
	chan := self joined: '#chan'.
	chan announcer on: IrkaMessage send: #msg: to: chanListener.
	self user: 'nick' said: 'msg 1 2' on: '#chan'.! !

!IrkaSessionTest methodsFor: 'tests' stamp: 'AttilaMagyar 9/23/2012 00:47'!
testThatItRecordsJoinedUsersInChannels
	| chan1 chan2 |			
	chan1 := self joined: '#chan1'.
	chan2 := self joined: '#chan2'.			
	self user: 'nick' joinedTo: '#chan1'.
	self assert: (self chan: chan1 hasUsers: {me. 'nick'}).
	self assert: (self chan: chan2 hasUsers: {me}).! !

!IrkaSessionTest methodsFor: 'tests' stamp: 'AttilaMagyar 9/23/2012 00:51'!
testThatItRecordsUsersAfterNickListReceived
	| chan |
	chan := self joined: '#chan'.
	session translated: (IrkaTranslated nickList: #(nick1 nick2) on: '#chan').
	self assert: (self chan: chan hasUsers: {'nick1'. 'nick2'}). ! !

!IrkaSessionTest methodsFor: 'tests' stamp: 'AttilaMagyar 9/26/2012 22:49'!
testThatItRegistersItself
	translator should receive: #translateRegisterUser:; with: me.
	session register: me.
! !

!IrkaSessionTest methodsFor: 'private' stamp: 'AttilaMagyar 9/22/2012 22:49'!
user: nick joinedTo: chan
	session translated: (IrkaTranslated joined: nick to: chan)! !

!IrkaSessionTest methodsFor: 'private' stamp: 'AttilaMagyar 9/22/2012 22:49'!
user: nick said: msg on: chan
	session translated: (IrkaTranslated message: msg fromNick: nick toChan: chan).! !

BabyMockTestCase subclass: #IrkaTranslatorTest
	instanceVariableNames: 'translator destination'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Irka-Test'!

!IrkaTranslatorTest methodsFor: 'running' stamp: 'AttilaMagyar 9/26/2012 22:30'!
setUp
	destination := context mock: 'connection'.
	translator := IrkaIrcTranslator toDestination: destination.
	context visualize: translator.! !

!IrkaTranslatorTest methodsFor: 'tests translate message to irc' stamp: 'AttilaMagyar 9/26/2012 22:30'!
testThatItTranslatesJoinMessage
	destination should receive: #send:; with: 'JOIN #chan'.
	translator translateJoin: '#chan'.! !

!IrkaTranslatorTest methodsFor: 'tests translate message to irc' stamp: 'AttilaMagyar 9/26/2012 22:30'!
testThatItTranslatesLeave
	destination should receive: #send:; with: 'PART #chan'.
	translator translateLeave: '#chan'.! !

!IrkaTranslatorTest methodsFor: 'tests translate message to irc' stamp: 'AttilaMagyar 9/26/2012 22:30'!
testThatItTranslatesMessage
	destination should receive: #send:; with: 'PRIVMSG #chan :msg 1 2'.
	translator translateMessage: 'msg 1 2' to: '#chan'.! !

!IrkaTranslatorTest methodsFor: 'tests translate irc to message' stamp: 'AttilaMagyar 9/23/2012 00:37'!
testThatItTranslatesNameListCommand
	| listener |
	listener := context mock: 'listener'.
	listener should receive: #namelist:; with: [:its | its chanName = '#chan' & its nicks = #(mynick othernick) ].
	translator announcer on: IrkaTranslatedNameList send: #namelist: to: listener.
	translator translateRaw: (RawLineReceived parse: ':server 353 mynick = #chan :mynick othernick').! !

!IrkaTranslatorTest methodsFor: 'tests translate irc to message' stamp: 'AttilaMagyar 9/26/2012 21:57'!
testThatItTranslatesNameListCommandWithAtSeparator
	| listener |
	listener := context mock: 'listener'.
	listener should receive: #namelist:; with: [:its | its chanName = '#chan' & its nicks = #('@mynick') ].
	translator announcer on: IrkaTranslatedNameList send: #namelist: to: listener.
	translator translateRaw: (RawLineReceived parse: ':server 353 mynick @ #chan :@mynick').! !

!IrkaTranslatorTest methodsFor: 'tests translate message to irc' stamp: 'AttilaMagyar 9/26/2012 22:30'!
testThatItTranslatesNickChange
	destination should receive: #send:; with: 'NICK newnick'.
	translator translateChangeNick: 'newnick'.! !

!IrkaTranslatorTest methodsFor: 'tests translate message to irc' stamp: 'AttilaMagyar 9/26/2012 22:30'!
testThatItTranslatesQuit
	destination should receive: #send:; with: 'QUIT'.
	destination should receive: #disconnect.
	translator translateQuit.! !

!IrkaTranslatorTest methodsFor: 'tests translate irc to message' stamp: 'AttilaMagyar 9/22/2012 23:53'!
testThatItTranslatesRawJoin
	| listener |
	listener := context mock: 'listener'.
	listener should receive: #joinChan:; with: [:it| it nick = 'nick!!host' & it chanName = '#chan' ].
	translator announcer on: IrkaTranslatedJoin send: #joinChan: to: listener.
	translator translateRaw: (RawLineReceived parse: 'nick!!host JOIN #chan').! !

!IrkaTranslatorTest methodsFor: 'tests translate irc to message' stamp: 'AttilaMagyar 9/22/2012 23:53'!
testThatItTranslatesRawPrivMsg
	| listener |
	listener := context mock: 'listener'.
	listener should receive: #msg:; with: [:it| it nick = 'nick!!host' & it chanName = '#chan' & it message = 'msg 1 2' ].
	translator announcer on: IrkaTranslatedChanMsg send: #msg: to: listener.
	translator translateRaw: (RawLineReceived parse: 'nick!!host PRIVMSG #chan :msg 1 2').! !

!IrkaTranslatorTest methodsFor: 'tests translate message to irc' stamp: 'AttilaMagyar 9/26/2012 22:30'!
testThatItTranslatesRegisterMessage
	destination should receive: #send:; with: 'NICK nick'.
	destination should receive: #send:; with: 'USER nick Server 0 :nick'.	
	translator translateRegisterUser: 'nick'.! !

BabyMockTestCase subclass: #IrkaUserTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Irka-Test'!

!IrkaUserTest methodsFor: 'tests' stamp: 'AttilaMagyar 9/25/2012 16:11'!
testThatItCanParseHostAndNick
	| user |
	user := IrkaUser parse: ':nick!!~ident@host.hu'.
	self assert: user nick equals: 'nick'.
	self assert: user ident equals: 'ident'.
	self assert: user host equals: 'host.hu'.! !

!IrkaUserTest methodsFor: 'tests' stamp: 'AttilaMagyar 9/25/2012 16:21'!
testThatItSignalsErrorIfHostIsIncorrect
	self should: [ IrkaUser parse: 'incorrect' ] raise: IrkaError.! !

Morph subclass: #IrkaMessagesMorph
	instanceVariableNames: 'messages'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Irka-GUI'!

!IrkaMessagesMorph methodsFor: 'adding' stamp: 'AttilaMagyar 9/24/2012 23:06'!
addMessage: message
	messages add: message! !

!IrkaMessagesMorph methodsFor: 'initialization' stamp: 'AttilaMagyar 9/26/2012 20:48'!
initialize 
	| listBox |
	super initialize.
	self hResizing: #spaceFill; vResizing: #spaceFill; color: Color transparent; layoutPolicy: TableLayout new.
	messages := IrkaListModel new.
	listBox := (PluggableListMorph 
				on: messages list: #list selected: #index changeSelected: #index: menu: #menu) 
				hResizing: #spaceFill;
				vResizing: #spaceFill;
				yourself.
	self addMorph: listBox.! !

Announcement subclass: #IrkaAction
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Irka-Core'!

!IrkaAction class methodsFor: 'instance creation' stamp: 'AttilaMagyar 9/22/2012 23:01'!
message: message on: chan from: user
	^ IrkaMessage new message: message on: chan from: user! !

IrkaAction subclass: #IrkaJoined
	instanceVariableNames: 'chan user'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Irka-Core'!

!IrkaJoined class methodsFor: 'instance creation' stamp: 'AttilaMagyar 9/22/2012 20:24'!
user: aUser to: aChan
	^ self new user: aUser chan: aChan! !

!IrkaJoined methodsFor: 'as yet unclassified' stamp: 'AttilaMagyar 9/22/2012 20:23'!
chan
	^ chan! !

!IrkaJoined methodsFor: 'as yet unclassified' stamp: 'AttilaMagyar 9/22/2012 20:26'!
user: aUser chan: aChan
	user := aUser.
	chan := aChan.! !

IrkaAction subclass: #IrkaMessage
	instanceVariableNames: 'chan user message'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Irka-Core'!

!IrkaMessage methodsFor: 'accessing' stamp: 'AttilaMagyar 9/22/2012 23:04'!
chan
	^ chan! !

!IrkaMessage methodsFor: 'accessing' stamp: 'AttilaMagyar 9/22/2012 23:04'!
message
	^ message! !

!IrkaMessage methodsFor: 'initialize-release' stamp: 'AttilaMagyar 9/22/2012 23:03'!
message: aMessage on: anIrkaChannel from: anIrkaUser
	message := aMessage.
	chan := anIrkaChannel.
	user := anIrkaUser.! !

!IrkaMessage methodsFor: 'accessing' stamp: 'AttilaMagyar 9/22/2012 23:04'!
user
	^ user! !

Announcement subclass: #IrkaTranslated
	instanceVariableNames: 'nick chanName'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Irka-Core'!

!IrkaTranslated class methodsFor: 'instance creation' stamp: 'AttilaMagyar 9/22/2012 23:56'!
joined: nick to: chanName
	^ IrkaTranslatedJoin new nick: nick; chanName: chanName; yourself! !

!IrkaTranslated class methodsFor: 'instance creation' stamp: 'AttilaMagyar 9/22/2012 23:56'!
message: msg fromNick: nick toChan: chanName
	^ IrkaTranslatedChanMsg new nick: nick; chanName: chanName; message: msg; yourself.! !

!IrkaTranslated class methodsFor: 'instance creation' stamp: 'AttilaMagyar 9/23/2012 00:14'!
nickList: anArray on: chanName
	^ IrkaTranslatedNameList new nicks: anArray; chanName: chanName; yourself.! !

!IrkaTranslated class methodsFor: 'instance creation' stamp: 'AttilaMagyar 9/25/2012 17:24'!
user: user changedNickTo: newNick
	^ IrkaTranslatedNickChange new user: user newNick: newNick; yourself.! !

!IrkaTranslated methodsFor: 'accessing' stamp: 'AttilaMagyar 9/22/2012 23:57'!
chanName
	^ chanName! !

!IrkaTranslated methodsFor: 'accessing' stamp: 'AttilaMagyar 9/22/2012 23:57'!
chanName: aByteSymbol 
	chanName := aByteSymbol.! !

!IrkaTranslated methodsFor: 'accessing' stamp: 'AttilaMagyar 9/15/2012 23:09'!
nick
	^ nick! !

!IrkaTranslated methodsFor: 'accessing' stamp: 'AttilaMagyar 9/15/2012 22:52'!
nick: aByteString 
	nick := aByteString ! !

IrkaTranslated subclass: #IrkaTranslatedChanMsg
	instanceVariableNames: 'message'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Irka-Core'!

!IrkaTranslatedChanMsg methodsFor: 'accessing' stamp: 'AttilaMagyar 9/17/2012 20:08'!
message
	^ message! !

!IrkaTranslatedChanMsg methodsFor: 'accessing' stamp: 'AttilaMagyar 9/16/2012 12:36'!
message: aByteString 
	message := aByteString ! !

!IrkaTranslatedChanMsg methodsFor: 'questions' stamp: 'AttilaMagyar 9/16/2012 12:39'!
whatHappened: questioner
	questioner chanMessageReceived: self.! !

IrkaTranslated subclass: #IrkaTranslatedJoin
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Irka-Core'!

!IrkaTranslatedJoin methodsFor: 'questions' stamp: 'AttilaMagyar 9/15/2012 23:08'!
whatHappened: questioner
	questioner joined: self! !

IrkaTranslated subclass: #IrkaTranslatedLeave
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Irka-Core'!

IrkaTranslated subclass: #IrkaTranslatedNameList
	instanceVariableNames: 'nicks'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Irka-Core'!

!IrkaTranslatedNameList methodsFor: 'accessing' stamp: 'AttilaMagyar 9/23/2012 00:33'!
nicks
	^ nicks! !

!IrkaTranslatedNameList methodsFor: 'accessing' stamp: 'AttilaMagyar 9/23/2012 00:15'!
nicks: anArray
	nicks := anArray ! !

!IrkaTranslatedNameList methodsFor: 'questions' stamp: 'AttilaMagyar 9/23/2012 00:46'!
whatHappened: questioner
	questioner nickListReceived: nicks on: chanName! !

IrkaTranslated subclass: #IrkaTranslatedNickChange
	instanceVariableNames: 'user'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Irka-Core'!

!IrkaTranslatedNickChange methodsFor: 'accessing' stamp: 'AttilaMagyar 9/25/2012 14:19'!
user
	^ user! !

!IrkaTranslatedNickChange methodsFor: 'accessing' stamp: 'AttilaMagyar 9/25/2012 13:47'!
user: aUser newNick: newNick
	user := aUser.
	nick := newNick.! !

!IrkaTranslatedNickChange methodsFor: 'questions' stamp: 'AttilaMagyar 9/26/2012 22:33'!
whatHappened: questioner
	questioner user: user changedNickTo: nick.! !

Announcement subclass: #RawLineReceived
	instanceVariableNames: 'sender command args'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Irka-Core'!

!RawLineReceived class methodsFor: 'instance creation' stamp: 'AttilaMagyar 9/17/2012 20:47'!
parse: line
	| args command tokens sender |
	args := line findTokens: ' '.
	tokens := args removeFirst: 2.
	sender := tokens at: 1.
	command := tokens at: 2.
	args := ' ' join: args.
	^ self new sender: sender; command: command; args: args; yourself.! !

!RawLineReceived methodsFor: 'initialize-release' stamp: 'AttilaMagyar 9/17/2012 20:51'!
args
	^ args! !

!RawLineReceived methodsFor: 'initialize-release' stamp: 'AttilaMagyar 9/17/2012 20:48'!
args: anArray
	args := anArray ! !

!RawLineReceived methodsFor: 'initialize-release' stamp: 'AttilaMagyar 9/17/2012 20:51'!
command
	^ command! !

!RawLineReceived methodsFor: 'initialize-release' stamp: 'AttilaMagyar 9/17/2012 20:48'!
command: aByteString
	command := aByteString ! !

!RawLineReceived methodsFor: 'initialize-release' stamp: 'AttilaMagyar 9/17/2012 20:51'!
sender
	^ sender! !

!RawLineReceived methodsFor: 'initialize-release' stamp: 'AttilaMagyar 9/17/2012 20:48'!
sender: aByteString 
	sender := aByteString! !

Object subclass: #Irka
	instanceVariableNames: 'connection translator session'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Irka-Core'!

!Irka class methodsFor: 'instance creation' stamp: 'AttilaMagyar 9/22/2012 18:42'!
host: hostName port: portNumber
	| hostIP |
	hostIP := NetNameResolver addressForName: hostName.
	^ self new ip: hostIP port: portNumber.! !

!Irka methodsFor: 'as yet unclassified' stamp: 'AttilaMagyar 9/26/2012 22:27'!
ip: hostIp port: portNumber
	connection := IrkaIrcConnection to: hostIp port: portNumber.
	session := IrkaSession new.
	translator := IrkaIrcTranslator toDestination: connection.
	session translator: translator.	
	connection announcer on: RawLineReceived send: #translateRaw: to: translator.	
	translator announcer on: IrkaTranslated send: #translated: to: session.		
	[ connection startProcessing ] forkNamed: 'Irka-IRC'.
	^ session.! !

!Irka methodsFor: 'as yet unclassified' stamp: 'AttilaMagyar 9/22/2012 18:40'!
stop
	connection disconnect.! !

Object subclass: #IrkaChannel
	instanceVariableNames: 'users chanName announcer translator'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Irka-Core'!

!IrkaChannel class methodsFor: 'instance creation' stamp: 'AttilaMagyar 9/23/2012 19:01'!
named: chanName translator: anIrkaTranslator
	^ self new chanName: chanName; translator: anIrkaTranslator.! !

!IrkaChannel methodsFor: 'adding' stamp: 'AttilaMagyar 9/15/2012 23:14'!
addUser: aUser
	users add: aUser.! !

!IrkaChannel methodsFor: 'accessing' stamp: 'AttilaMagyar 9/16/2012 12:42'!
announcer 
	^ announcer! !

!IrkaChannel methodsFor: 'accessing' stamp: 'AttilaMagyar 9/15/2012 23:44'!
chanName
	^ chanName! !

!IrkaChannel methodsFor: 'initialize-release' stamp: 'AttilaMagyar 9/15/2012 23:44'!
chanName: aByteSymbol
	chanName := aByteSymbol! !

!IrkaChannel methodsFor: 'initialize-release' stamp: 'AttilaMagyar 9/16/2012 12:45'!
initialize 
	super initialize.
	users := OrderedCollection new.
	announcer := Announcer new.! !

!IrkaChannel methodsFor: 'commands' stamp: 'AttilaMagyar 9/23/2012 19:22'!
leave
	translator translateLeave: chanName.! !

!IrkaChannel methodsFor: 'commands' stamp: 'AttilaMagyar 9/23/2012 19:09'!
message: message
	translator translateMessage: message to: chanName.! !

!IrkaChannel methodsFor: 'events' stamp: 'AttilaMagyar 9/22/2012 23:00'!
message: msg from: user
	announcer announce: (IrkaAction message: msg on: self from: user).! !

!IrkaChannel methodsFor: 'printing' stamp: 'AttilaMagyar 9/24/2012 23:16'!
printString 
	^ chanName ! !

!IrkaChannel methodsFor: 'accessing' stamp: 'AttilaMagyar 9/23/2012 19:01'!
translator: anIrkaTranslator 
	translator := anIrkaTranslator! !

!IrkaChannel methodsFor: 'accessing' stamp: 'AttilaMagyar 9/15/2012 23:59'!
users
	^ users! !

!IrkaChannel methodsFor: 'adding' stamp: 'AttilaMagyar 9/23/2012 00:55'!
users: arrayOfUsers
	users removeAll.
	arrayOfUsers do: [:eachUser | self addUser: eachUser ].! !

Object subclass: #IrkaIrcConnection
	instanceVariableNames: 'sockStream announcer'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Irka-Core'!

!IrkaIrcConnection class methodsFor: 'instance creation' stamp: 'AttilaMagyar 9/22/2012 18:35'!
to: hostIp port: portNumber
	^ self new connectTo: hostIp port: portNumber ! !

!IrkaIrcConnection methodsFor: 'accessing' stamp: 'AttilaMagyar 9/17/2012 20:34'!
announcer
	^ announcer! !

!IrkaIrcConnection methodsFor: 'control' stamp: 'AttilaMagyar 9/17/2012 20:36'!
connectTo: hostIP port: portNumber
	sockStream := SocketStream openConnectionToHost: hostIP port: portNumber.
	sockStream noTimeout.! !

!IrkaIrcConnection methodsFor: 'control' stamp: 'AttilaMagyar 9/17/2012 20:32'!
disconnect
	sockStream close
! !

!IrkaIrcConnection methodsFor: 'initialize-release' stamp: 'AttilaMagyar 9/17/2012 20:34'!
initialize 
	super initialize.
	announcer := Announcer new.! !

!IrkaIrcConnection methodsFor: 'private' stamp: 'AttilaMagyar 9/26/2012 20:53'!
readAndAnnounceLine: line
	| tokens |
	line hasIrcPrefix
		ifTrue: [ 
			announcer announce: (RawLineReceived parse: line) ]
		ifFalse: [ 
			(line withoutIrcPrefix beginsWith: 'PING')
				ifTrue: [ self send: (line copyReplaceAll: 'PING' with: 'PONG') ] ]! !

!IrkaIrcConnection methodsFor: 'control' stamp: 'AttilaMagyar 9/17/2012 20:32'!
send: command
	sockStream sendCommand: command! !

!IrkaIrcConnection methodsFor: 'control' stamp: 'AttilaMagyar 9/26/2012 20:54'!
startProcessing
	| line |	
	[ sockStream isConnected ]
		whileTrue: [
			line := sockStream nextLine.
			self readAndAnnounceLine: line.
			Transcript show: line; cr ]
	! !

Object subclass: #IrkaIrcTranslator
	instanceVariableNames: 'announcer destination'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Irka-Core'!

!IrkaIrcTranslator class methodsFor: 'instance creation' stamp: 'AttilaMagyar 9/26/2012 22:29'!
toDestination: destination
	^ self new destination: destination.! !

!IrkaIrcTranslator methodsFor: 'accessing' stamp: 'AttilaMagyar 9/17/2012 21:08'!
announcer 
	^ announcer ! !

!IrkaIrcTranslator methodsFor: 'parsing' stamp: 'AttilaMagyar 9/26/2012 21:13'!
cmd353: rawLineReceived
	"pharobot4 = #takarodjki :pharobot4 kalohu giszo zeroflag"
	| args chan nicks chanStart chanEnd |
	args := rawLineReceived args.
	chanStart := (self findSeparatorInNickList: args) + 3.
	chanEnd := args findString: ' :' startingAt: chanStart.
	chan := args copyFrom: chanStart to: chanEnd - 1.
	nicks := args copyFrom: chanEnd + 2 to: args size.
	announcer announce: (IrkaTranslated nickList: (nicks findTokens: ' ') asArray on: chan)! !

!IrkaIrcTranslator methodsFor: 'parsing' stamp: 'AttilaMagyar 9/23/2012 00:02'!
cmdJOIN: rawLineReceived
	"command=JOIN args=:#funcoder2"
	| chan user |
	chan := rawLineReceived args.
	user := rawLineReceived sender.
	announcer announce: (IrkaTranslated joined: user to: chan).! !

!IrkaIrcTranslator methodsFor: 'parsing' stamp: 'AttilaMagyar 9/25/2012 17:24'!
cmdNICK: rawLineReceived
	":zero88!!~zero88@zeus.csko.hu NICK :zero99"
	| parts chan nicks sep newNick host |
	newNick := rawLineReceived args withoutIrcPrefix.
	host := rawLineReceived sender withoutIrcPrefix.

	announcer announce: (IrkaTranslated user: (IrkaUser parse: host) changedNickTo: newNick).! !

!IrkaIrcTranslator methodsFor: 'parsing' stamp: 'AttilaMagyar 9/23/2012 00:02'!
cmdPRIVMSG: rawLineReceived
	"command=PRIVMSG args=#funcoder :1 2 3"
	| parts chan msg |
	parts := rawLineReceived args findTokens: ' '.
	chan := parts removeFirst.
	msg := (' ' join: parts) allButFirst: 1.
	announcer announce: (IrkaTranslated message: msg fromNick: rawLineReceived sender toChan: chan).! !

!IrkaIrcTranslator methodsFor: 'initialize-release' stamp: 'AttilaMagyar 9/26/2012 22:28'!
destination: dest
	destination := dest! !

!IrkaIrcTranslator methodsFor: 'private' stamp: 'AttilaMagyar 9/26/2012 21:14'!
findSeparatorInNickList: line
	| idx |
	idx := line findString: ' = '.
	idx isZero
		ifTrue: [ idx := line findString: ' @ ' ].
	idx isZero 
		ifTrue: [ IrkaError signal: 'Invalid nicklist format ' , line ].
	^ idx! !

!IrkaIrcTranslator methodsFor: 'initialize-release' stamp: 'AttilaMagyar 9/17/2012 21:07'!
initialize 
	super initialize.
	announcer := Announcer new.! !

!IrkaIrcTranslator methodsFor: 'translating message to irc' stamp: 'AttilaMagyar 9/26/2012 22:26'!
translateChangeNick: newNick 
	destination send: 'NICK ', newNick! !

!IrkaIrcTranslator methodsFor: 'translating message to irc' stamp: 'AttilaMagyar 9/26/2012 22:26'!
translateJoin: chanName
	destination send: 'JOIN ', chanName! !

!IrkaIrcTranslator methodsFor: 'translating message to irc' stamp: 'AttilaMagyar 9/26/2012 22:26'!
translateLeave: chanName 
	destination send: 'PART ', chanName.! !

!IrkaIrcTranslator methodsFor: 'translating message to irc' stamp: 'AttilaMagyar 9/26/2012 22:26'!
translateMessage: message to: target
	destination send: 'PRIVMSG ', target asString, ' :', message asString.! !

!IrkaIrcTranslator methodsFor: 'translating message to irc' stamp: 'AttilaMagyar 9/26/2012 22:26'!
translateQuit
	destination send: 'QUIT'; disconnect.! !

!IrkaIrcTranslator methodsFor: 'translating irc to message' stamp: 'AttilaMagyar 9/23/2012 00:02'!
translateRaw: aRawLineReceived	
	| msg |
	msg := ('cmd', aRawLineReceived command, ':') asSymbol.
	(self respondsTo: msg) ifTrue: [
		self perform: msg with: aRawLineReceived.
	]! !

!IrkaIrcTranslator methodsFor: 'translating message to irc' stamp: 'AttilaMagyar 9/26/2012 22:26'!
translateRegisterUser: aNick
	destination send: 'NICK ', aNick;
					send: 'USER ', aNick, ' Server 0 :', aNick.! !

Object subclass: #IrkaListModel
	instanceVariableNames: 'list index'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Irka-GUI'!

!IrkaListModel methodsFor: 'accessing' stamp: 'AttilaMagyar 9/23/2012 20:55'!
add: element
	list add: element.
	self changed: #list.! !

!IrkaListModel methodsFor: 'accessing' stamp: 'AttilaMagyar 9/23/2012 21:17'!
clean
	list removeAll.
	self changed: #list.! !

!IrkaListModel methodsFor: 'accessing' stamp: 'AttilaMagyar 9/23/2012 20:30'!
index
	^ index! !

!IrkaListModel methodsFor: 'accessing' stamp: 'AttilaMagyar 9/23/2012 20:36'!
index: anInteger
	index := anInteger.
	self changed: #index.! !

!IrkaListModel methodsFor: 'initialization' stamp: 'AttilaMagyar 9/23/2012 20:33'!
initialize 
	super initialize.
	index := 0.
	list := OrderedCollection new.! !

!IrkaListModel methodsFor: 'accessing' stamp: 'AttilaMagyar 9/23/2012 20:30'!
list
	^ list! !

!IrkaListModel methodsFor: 'accessing' stamp: 'AttilaMagyar 9/23/2012 20:32'!
menu
	^ nil! !

Object subclass: #IrkaSession
	instanceVariableNames: 'translator channels nick announcer'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Irka-Core'!

!IrkaSession methodsFor: 'accessing' stamp: 'AttilaMagyar 9/22/2012 19:31'!
announcer 
	^ announcer ! !

!IrkaSession methodsFor: 'translated events' stamp: 'AttilaMagyar 9/24/2012 23:23'!
chanMessageReceived: anIrkaTranslatedChanMsg
	| chan chanName |
	chanName := anIrkaTranslatedChanMsg chanName.
	chan := channels at: chanName ifAbsent: [ ^ nil. "Error signal: 'Chan not found ', chanName" ].	 "TODO priv msg"
	chan message: anIrkaTranslatedChanMsg message from: anIrkaTranslatedChanMsg nick.! !

!IrkaSession methodsFor: 'commands' stamp: 'AttilaMagyar 9/23/2012 18:50'!
changeNick: newNick 
	translator translateChangeNick: newNick.! !

!IrkaSession methodsFor: 'commands' stamp: 'AttilaMagyar 9/26/2012 19:30'!
close
	translator translateQuit.! !

!IrkaSession methodsFor: 'initialize-release' stamp: 'AttilaMagyar 9/22/2012 19:31'!
initialize 
	super initialize.
	channels := Dictionary new.
	announcer := Announcer new.
	nick := 'IrkaSessionNick'.! !

!IrkaSession methodsFor: 'commands' stamp: 'AttilaMagyar 9/22/2012 20:21'!
joinChan: chanName 
	translator translateJoin: chanName asString.! !

!IrkaSession methodsFor: 'translated events' stamp: 'AttilaMagyar 9/23/2012 19:10'!
joined: anIrkaTranslatedJoin
	| chan chanName |	
	chanName := anIrkaTranslatedJoin chanName.
	chan := channels at: chanName ifAbsent: [ IrkaChannel named: chanName translator: translator ].
	chan addUser: anIrkaTranslatedJoin nick.
	channels at: chanName put: chan.
	announcer announce: (IrkaJoined user: anIrkaTranslatedJoin nick to: chan).! !

!IrkaSession methodsFor: 'translated events' stamp: 'AttilaMagyar 9/23/2012 00:54'!
nickListReceived: arrayOfNicks on: chanName
	| chan |
	chan := channels at: chanName ifAbsent: [ Error signal: 'Chan not found ', chanName ].	
	chan users: arrayOfNicks.
	! !

!IrkaSession methodsFor: 'commands' stamp: 'AttilaMagyar 9/26/2012 22:51'!
register: aNick
	translator translateRegisterUser: aNick.! !

!IrkaSession methodsFor: 'translated events' stamp: 'AttilaMagyar 9/22/2012 22:49'!
translated: anIrkaTranslated
	anIrkaTranslated whatHappened: self.! !

!IrkaSession methodsFor: 'initialize-release' stamp: 'AttilaMagyar 9/15/2012 22:40'!
translator: anIrkaTranslator
	translator := anIrkaTranslator.! !

!IrkaSession methodsFor: 'translated events' stamp: 'AttilaMagyar 9/26/2012 22:51'!
user: user changedNickTo: newNick
	nick ifNil:  [ nick := newNick ].! !

Object subclass: #IrkaUser
	instanceVariableNames: 'nick host ident'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Irka-Core'!

!IrkaUser class methodsFor: 'instance creation' stamp: 'AttilaMagyar 9/25/2012 16:03'!
parse: aString
	^ self new parse: aString ! !

!IrkaUser methodsFor: 'comparing' stamp: 'AttilaMagyar 9/25/2012 17:20'!
= anObject
	"Answer whether the receiver and anObject represent the same object."

	self == anObject
		ifTrue: [ ^ true ].
	self class = anObject class
		ifFalse: [ ^ false ].
	^ nick = anObject nick and: [ host = anObject host and: [ ident = anObject ident ] ]! !

!IrkaUser methodsFor: 'private' stamp: 'AttilaMagyar 9/25/2012 16:52'!
findIdentHostSeparator: hostMask
	| idx |
	idx := hostMask findString: '@' startingAt: (self findNickIdentSeparator: hostMask).
	idx isZero
		ifTrue: [ IrkaError signal: 'Invalid host format ' , hostMask ].
	^ idx	! !

!IrkaUser methodsFor: 'private' stamp: 'AttilaMagyar 9/25/2012 16:46'!
findNickIdentSeparator: hostMask
	| idx |
	idx := hostMask findString: '!!'.
	idx isZero
		ifTrue: [ IrkaError signal: 'Invalid host format ' , hostMask ].
	^ idx! !

!IrkaUser methodsFor: 'comparing' stamp: 'AttilaMagyar 9/25/2012 17:20'!
hash
	"Answer an integer value that is related to the identity of the receiver."

	^ nick hash bitXor: (host hash bitXor: ident hash)! !

!IrkaUser methodsFor: 'accessing' stamp: 'AttilaMagyar 9/25/2012 16:06'!
host
	^ host! !

!IrkaUser methodsFor: 'accessing' stamp: 'AttilaMagyar 9/25/2012 16:16'!
ident
	^ ident! !

!IrkaUser methodsFor: 'accessing' stamp: 'AttilaMagyar 9/25/2012 16:05'!
nick
	^ nick! !

!IrkaUser methodsFor: 'initialize-release' stamp: 'AttilaMagyar 9/25/2012 16:42'!
parse: aString
	"parse hostmasks like :nick!!~ident@host.hu"
	| hostMask |
	hostMask := aString withoutIrcPrefix.
	nick := self parseNick: hostMask.
	ident := self parseIdent: hostMask.
	host := self parseHost: hostMask.! !

!IrkaUser methodsFor: 'private' stamp: 'AttilaMagyar 9/25/2012 17:12'!
parseHost: hostMask
	| hostStart |
	hostStart := (self findIdentHostSeparator: hostMask) +1.
	^ hostMask copyFrom: hostStart to: hostMask size.! !

!IrkaUser methodsFor: 'private' stamp: 'AttilaMagyar 9/25/2012 17:13'!
parseIdent: hostMask
	| identStart identEnd |
	identStart := (self findNickIdentSeparator: hostMask) + 1.
	identEnd := (self findIdentHostSeparator: hostMask) - 1.
	^ self trimTilde: (hostMask copyFrom: identStart to: identEnd)! !

!IrkaUser methodsFor: 'private' stamp: 'AttilaMagyar 9/25/2012 16:48'!
parseNick: hostMask
	^ hostMask copyFrom: 1 to: (self findNickIdentSeparator: hostMask) - 1! !

!IrkaUser methodsFor: 'private' stamp: 'AttilaMagyar 9/25/2012 17:13'!
trimTilde: fullIdent
	^ (fullIdent beginsWith: '~')
		ifTrue: [ fullIdent allButFirst: 1 ]
		ifFalse: [ fullIdent ]! !

Error subclass: #IrkaError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Irka-Core'!

!String methodsFor: '*irka' stamp: 'AttilaMagyar 9/25/2012 14:36'!
hasIrcPrefix
	^ self beginsWith: ':'! !

!String methodsFor: '*irka' stamp: 'AttilaMagyar 9/25/2012 14:37'!
withoutIrcPrefix
	^ (self beginsWith: ':') ifTrue: [ self allButFirst: 1 ] ifFalse: [ self ]! !
